#include "dll_sys_access_wrap.h"
#include "dll_SystemAccess.h"
#include "vcs_DebugMalloc.h"
#include "vcsprint.h"
#include "stdarg.h"
#include "stdio.h"
#include <new>


static C_DllSystemAccess* s_pSystemAccess = NULL;


bool SetSystemAccess(C_DllSystemAccess &SystemAccess)
{
	C_DllSystemAccess::t_Version Version = SystemAccess.getVersion();
	if(Version.Major != DLL_SYSTEM_ACCESS_MAJOR_VERSION)
	{
		return false;
	}
	
	s_pSystemAccess = &SystemAccess;
	return true;
}


static inline void* SystemAccessMalloc(size_t size)
{
	if(s_pSystemAccess)
	{
		return s_pSystemAccess->malloc(size);
	}
	
	return NULL;
}

static inline void SystemAccessDelete(void* p)
{
	if(s_pSystemAccess && p)
	{
		s_pSystemAccess->free(p);
	}
}


void* operator new(size_t size)
{
	return SystemAccessMalloc(size);
}

void* operator new[](size_t size)
{
	return SystemAccessMalloc(size);
}

void* operator new(size_t size, const std::nothrow_t&)
{
	return SystemAccessMalloc(size);
}

void* operator new[](size_t size, const std::nothrow_t&)
{
	return SystemAccessMalloc(size);
}

void operator delete(void* p)
{
	SystemAccessDelete(p);
}

void operator delete[](void* p)
{
	SystemAccessDelete(p);
}

void operator delete(void* p, const std::nothrow_t&)
{
	SystemAccessDelete(p);
}

void operator delete[](void* p, const std::nothrow_t&)
{
	SystemAccessDelete(p);
}

extern "C"
void *vcs_malloc_ex(OCTET *path, int line, size_t size, int align_size, VCS_HEAP heap, DWORD flags)
{
	return SystemAccessMalloc(size);
}

extern "C"
void vcs_free(OCTET *path, int line, void *p)
{
	SystemAccessDelete(p);
}

extern "C"
int vcs_do_printout(unsigned int id)
{
	t_DLL_PRINT_ID DllId = DLL_PRINT_ID_LAST;
	
	switch(id)
	{
		case STDTERM:
			DllId = DLL_STDTERM;
			break;
		case IMAGEPIPE:
			DllId = DLL_IMAGEPIPE;
			break;
		case SYSCON:
			DllId = DLL_SYSCON;
			break;
		case AUTOFOCUS:
			DllId = DLL_AUTOFOCUS;
			break;
		case SXI_DLL:
			DllId = DLL_SXI;
			break;
		case VIPROC:
			DllId = DLL_VCA;
			break;
		default:
			break;
	}
	
	if(s_pSystemAccess && (s_pSystemAccess->printf_id(DllId, NULL) == 0))
	{
		return 1;
	}
	
	return 0;
}

extern "C"
int vcs_sprintfex(char *buf, size_t max_bytes, const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	int result = vsnprintf(buf, max_bytes, fmt, ap);
	va_end(ap);
	
	return result;
}

extern "C"
int TERM_PRINT(char const* fmt, ...)
{
	char aBuf[512];
	
	va_list ap;
	va_start(ap, fmt);
	vsnprintf(aBuf, sizeof(aBuf), fmt, ap);
	va_end(ap);
	
	int result = 0;
	
	if(s_pSystemAccess)
	{
		result = s_pSystemAccess->printf_id(DLL_STDTERM, aBuf);
	}
	
	return result;
}

extern "C"
void tx_thread_sleep(DWORD dw10MS)
{
	if(s_pSystemAccess)
	{
		s_pSystemAccess->sleepThread(dw10MS);
	}
}

extern "C"
DWORD GetTime90KHZ()
{
	DWORD Time90KHZ = 0;

	if(s_pSystemAccess)
	{
		Time90KHZ = s_pSystemAccess->GetTime90KHz();
	}

	return Time90KHZ;
}
