#ifndef VCS_DEBUG_MALLOC_H_INCLUDED
#define VCS_DEBUG_MALLOC_H_INCLUDED

#include "vcsIntTypes.h"



typedef enum VCS_HEAP
{
	VCS_HEAP_STATIC,
	VCS_HEAP_GLOBAL,
	VCS_HEAP_IMAGE,
	VCS_HEAP_VIPROC,
	VCS_HEAP_ISRAM,

	VCS_HEAP_COUNT  // last (for all normal heaps)

} VCS_HEAP;


#define DMALLOC(n)                vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,8,VCS_HEAP_GLOBAL,0)
#define DMALLOC_FLAG(n,f)         vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,8,VCS_HEAP_GLOBAL,f)
#define DMALLOC_ALIGN(n,a)        vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,a,VCS_HEAP_GLOBAL,0)
#define DMALLOC_ALIGN_FLAG(n,a,f) vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,a,VCS_HEAP_GLOBAL,f)
#define DMALLOC_HEAP(n,h)         vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,8,h              ,0)
#define DMALLOC_ALIGN_HEAP(n,a,h) vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,a,h              ,0)
#define DFREE(p)                  vcs_free     ((OCTET *)__FILE__,__LINE__,p)
#define DMALLOC_HEAP_FLAG(n,h,f)  vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,8,h              ,f)
#define DMALLOC_EX(n,a,h,f)       vcs_malloc_ex((OCTET *)__FILE__,__LINE__,n,a,h              ,f)
#define DCALLOC(n,s)              vcs_calloc_ex((OCTET *)__FILE__,__LINE__,n,s,8,VCS_HEAP_GLOBAL,0)


#ifdef __cplusplus
extern "C" {
#endif


#define VCS_MALLOC_FLAG_GLOBAL_NEW     0x00000001
#define VCS_MALLOC_FLAG_SILENT_ON_FAIL 0x00000002
#define VCS_MALLOC_FLAG_FORCE_HEAP     0x00000004
#define VCS_MALLOC_FLAG_UNCACHED       0x00000008
#define VCS_MALLOC_FLAG_STRGLY_ORD     0x00000010  // ARMv7 Memory-Type e.g. for DMA use
#define VCS_MALLOC_FLAG_ONLY_L1_CACHED 0x00000020  // ARMv7 L1 Cached L2 Unchached ARMv7 Only
#define VCS_MALLOC_FLAG_DEVICE         0x00000040  // ARMv7 Unchached but not strongly ordered ARMv7 Only

void *vcs_malloc_ex(OCTET *path, int line, size_t size, int align_size, VCS_HEAP heap, DWORD flags);
void *vcs_calloc_ex(OCTET *path, int line, int num, size_t size, int align_size, VCS_HEAP heap, DWORD flags);
void  vcs_free     (OCTET *path, int line, void *p);


#ifdef __cplusplus
} // end extern "C"

#if defined(OS_ANDROID) || defined(OS_WIN)
 #define VCS_NOEXCEPT noexcept
#else
 #define VCS_NOEXCEPT
#endif


// TH: implement the operators directly
#define VCS_DEBUG_NEW_OPERATORS                                                   \
	void *operator new(size_t n) VCS_NOEXCEPT                                     \
	{                                                                             \
	   return DMALLOC(n);                                                         \
	}                                                                             \
	void *operator new(size_t n, VCS_HEAP h) VCS_NOEXCEPT                         \
	{                                                                             \
	   return DMALLOC_HEAP(n,h);                                                  \
	}                                                                             \
	void *operator new(size_t n, char const *f, DWORD l) VCS_NOEXCEPT             \
	{                                                                             \
	   return vcs_malloc_ex((OCTET *)f,l,n,8,VCS_HEAP_GLOBAL,0);                  \
	}                                                                             \
	void *operator new(size_t n, VCS_HEAP h, char const *f, DWORD l) VCS_NOEXCEPT  \
	{                                                                             \
	   return vcs_malloc_ex((OCTET *)f,l,n,8,h,0);                                \
	}                                                                             \
	void *operator new[](size_t n) VCS_NOEXCEPT                                   \
	{                                                                             \
	   return DMALLOC(n);                                                         \
	}                                                                             \
	void *operator new[](size_t n, VCS_HEAP h) VCS_NOEXCEPT                       \
	{                                                                             \
	   return DMALLOC_HEAP(n,h);                                                  \
	}                                                                             \
	void *operator new[](size_t n, char const *f, DWORD l) VCS_NOEXCEPT           \
	{                                                                             \
	   return vcs_malloc_ex((OCTET *)f,l,n,8,VCS_HEAP_GLOBAL,0);                  \
	}                                                                             \
	void *operator new[](size_t n, VCS_HEAP h, char const *f, DWORD l)VCS_NOEXCEPT \
	{                                                                             \
	   return vcs_malloc_ex((OCTET *)f,l,n,8,h,0);                                \
	}

#define VCS_DEBUG_DELETE_OPERATORS                                               \
	void operator delete(void *p)                                                 \
	{                                                                             \
	   DFREE(p);                                                                  \
	}                                                                             \
	void operator delete(void *p, VCS_HEAP)                                       \
	{                                                                             \
	   DFREE(p);                                                                  \
	}                                                                             \
	void operator delete(void *p, char const *f, DWORD l)                         \
	{                                                                             \
	   vcs_free((OCTET *)f,l,p);                                                  \
	}                                                                             \
	void operator delete(void *p, VCS_HEAP, char const *f, DWORD l)               \
	{                                                                             \
	   vcs_free((OCTET *)f,l,p);                                                  \
	}                                                                             \
	void operator delete[](void *p)                                               \
	{                                                                             \
	   DFREE(p);                                                                  \
	}                                                                             \
	void operator delete[](void *p, VCS_HEAP)                                     \
	{                                                                             \
	   DFREE(p);                                                                  \
	}                                                                             \
	void operator delete[](void *p, char const *f, DWORD l)                       \
	{                                                                             \
	   vcs_free((OCTET *)f,l,p);                                                  \
	}                                                                             \
	void operator delete[](void *p, VCS_HEAP, char const *f, DWORD l)             \
	{                                                                             \
	   vcs_free((OCTET *)f,l,p);                                                  \
	}

#define VCS_DEBUG_MALLOC_OPERATORS                                               \
	VCS_DEBUG_NEW_OPERATORS                                                       \
	VCS_DEBUG_DELETE_OPERATORS


#endif // __cplusplus


#endif // VCS_DEBUG_MALLOC_H_INCLUDED
