#ifndef VCS_PRINT_H
#define VCS_PRINT_H

#include "vcs_DebugMalloc.h"
#include <stdarg.h> /* for va_list */

#include "syslog2.h"


#ifdef __cplusplus
extern "C" {
#endif

int TERM_VPRINT(char const* pFormat, va_list args);
int TERM_PRINT (char const* pFormat, ...);
int TERM_PUTSTR(char const* sz, DWORD len);
#define PRINTOUT_FUNCTION TERM_PRINT

int vcs_vsprintf(char *buf, size_t max_bytes, const char *fmt, va_list args);

int HexToString(DWORD StartAddr, DWORD BytesPerLine, const OCTET* pData, DWORD nDataLen, char* sz, DWORD max_bytes);
BOOL HexPrint(DWORD StartAddr, DWORD BytesPerLine, const OCTET* pData, DWORD nDataLen);
#define VIEW_HEX(a, b) HexPrint(0, 8, a, b)

void LogFileInit(const char* szPath, const char* szFilename);
void LogFilePut(const char* sz, DWORD len);


//this will be checked on every printout
int      vcs_do_printout(unsigned int id);
int      vcs_do_printout_enc(unsigned int id);

//visibility can be setup by the user
void        PRINT_Show(unsigned int id, int onoff);
void        PRINT_Show_Range(unsigned int id1, unsigned int id2, int onoff);
void        PRINT_ShowToggle(unsigned int id);
int         PRINT_Visible(unsigned int id);
const char *PRINT_GetIdName(unsigned int id);
const char *PRINT_GetInternalName(unsigned int id);
const char *PRINT_GetDescription(unsigned int id);
int         PRINT_GetStatus(unsigned int id);
int         PRINT_GetIdByIndex(int idx);

// To get and set SetHashunsigned ints for saving settings to config
void PRINT_SetHashDwords( unsigned int* HashDw );
void PRINT_GetHashDwords( unsigned int* HashDw );
int PRINT_SetEnabledPrintLevelsStr(const char *pStr);
int PRINT_GetEnabledPrintLevelsStr(char *pStr, int MaxStrLen);


//prints a long string into multiple small bulks (e.g. 256 bytes)
int PRINT_LongString(const char *pLongStr);

//prints a string to OSD
int PRINT_OsdString(const char *pOsdStr);


#ifdef VCS_APP0
	#define CHECK_CHANNEL(id) (id==STDTERM)
#else
	#define CHECK_CHANNEL(id) (vcs_do_printout(id))
#endif


#define STD_PRINT0(id,s)                                    do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s);}}while(0)
#define STD_PRINT1(id,s,a)                                  do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a);}}while(0)
#define STD_PRINT2(id,s,a,b)                                do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b);}}while(0)
#define STD_PRINT3(id,s,a,b,c)                              do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c);}}while(0)
#define STD_PRINT4(id,s,a,b,c,d)                            do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d);}}while(0)
#define STD_PRINT5(id,s,a,b,c,d,e)                          do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e);}}while(0)
#define STD_PRINT6(id,s,a,b,c,d,e,f)                        do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f);}}while(0)
#define STD_PRINT7(id,s,a,b,c,d,e,f,g)                      do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g);}}while(0)
#define STD_PRINT8(id,s,a,b,c,d,e,f,g,h)                    do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h);}}while(0)
#define STD_PRINT9(id,s,a,b,c,d,e,f,g,h,i)                  do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i);}}while(0)
#define STD_PRINT10(id,s,a,b,c,d,e,f,g,h,i,j)               do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j);}}while(0)
#define STD_PRINT11(id,s,a,b,c,d,e,f,g,h,i,j,k)             do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k);}}while(0)
#define STD_PRINT12(id,s,a,b,c,d,e,f,g,h,i,j,k,l)           do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k,l);}}while(0)
#define STD_PRINT13(id,s,a,b,c,d,e,f,g,h,i,j,k,l,m)         do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k,l,m);}}while(0)
#define STD_PRINT14(id,s,a,b,c,d,e,f,g,h,i,j,k,l,m,n)       do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k,l,m,n);}}while(0)
#define STD_PRINT15(id,s,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o)     do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o);}}while(0)
#define STD_PRINT16(id,s,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p)   do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(s,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p);}}while(0)
#define HEX_PRINT(id,a,b)                                   do{if(CHECK_CHANNEL(id)){VIEW_HEX(a,b);}}while(0)
#define STD_PRINT_STR(id, s)                                do{if(CHECK_CHANNEL(id)){PRINT_LongString(s);}}while(0)
#define STD_PRINT_OSD(id, s)                                do{if(CHECK_CHANNEL(id)){PRINT_OsdString(s);}}while(0)

#if defined(GHS_ARM_BIOS) || defined(OS_ANDROID) || defined(OS_LINUX)
#define STD_PRINTX(id, format, arg...)                      do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(format, ## arg);}}while(0)
#else
#define STD_PRINTX(id, format, ...)                         do{if(CHECK_CHANNEL(id)){PRINTOUT_FUNCTION(format, __VA_ARGS__);}}while(0)
#endif


#define     NBR_OF_HASH_DWORDS   16
#define     BITS_PER_DWORD       32
#define     LOG_BITS_PER_DWORD   5  //32 = 2^5
#define     MASK_BITS_PER_DWORD  ((BITS_PER_DWORD) - 1) //32 - 1 = 31 = 0x1F

#define     MAX_HASH_IDS         (BITS_PER_DWORD * NBR_OF_HASH_DWORDS)
//->8 dwords -> 256 IDs max.

#define     TEMP_OFF_FLAG 0x8000

//Used IDs
enum VCS_Print_IDs
{
	STDTERM                      =   0,
	ISDN_PRINT                   =   1,
	TM_VID_ENC_TERM              =   2,
	TRANSPARENT_DATA             =   3,
	TCP_IP                       =   4,
	HTTP                         =   5,
	RCP                          =   6,
	ALARM_PRINT                  =   7,
	HDD_PRINT                    =   8,
	WLAN_PRINT                   =   9,
	TIME_PRINT                   =  10,
	CONFIG                       =  11,
	TM_VID_DEC_TERM              =  12,
	RECEIVER                     =  13,
	RCP_CONNECT_VERBOSE          =  14,
	NFS                          =  15,
	SNMP_PRINT                   =  16,
	RTL8110                      =  17,
	SNDVID                       =  18,
	VI_TVP5145                   =  19,
	VCS_DYNDNS                   =  20,
	PPP_PRINT                    =  21,
	ALARM_CONN_PRINT             =  22,
	VIDEOIN                      =  23,
	VIDEOOUT                     =  24,
	IGMP                         =  25,
	MPEG2_ENC                    =  26,
	RCP_SRV                      =  27,
	RCP_CLT                      =  28,
	RCP_CON                      =  29,
	RCP_UDP                      =  30,
	RCP_COMMAND                  =  31,
	RCP_AUTH                     =  32,
	RCP_UTIL                     =  33,
	RCP_CON_RETRIGGER            =  34,
	RCP_AUTH_DBG                 =  35,
	RCP_TASK_DBG                 =  36,
	MENU_DBG                     =  37,
	HDD_REC                      =  38,
	HDD_REP                      =  39,
	RCP_MSG                      =  40,
	HDDCACHE                     =  41,
	HDDPM                        =  42,
	USB_PRINT                    =  43,
	TERM_REPLAY                  =  44,
	PICINFO                      =  45,
	JPEG                         =  46,
	PARAM                        =  47,
	SYSTEM                       =  48,
	MULTICAST                    =  49,
	ARP                          =  50,
	OVERLAY                      =  51,
	SHAREBUF                     =  52,
	MODEM                        =  53,
	BREAKPOINT_DBG               =  54,
	ADD_IO_BOARD                 =  55,
	G711_PRINT                   =  56,
	NAT                          =  57,
	COPROC                       =  58,
	BLOCKPOOL                    =  59,
	QUEUE                        =  60,
	RCPCMD_VERBOSE               =  61,
	RCP_COMMAND_LOCAL_INT        =  62,
	BSP_CLT                      =  63,
	RCP_PROXY                    =  64,
	RCP_BCLT                     =  65,
	VGAOUT                       =  66,
	DISPLAY                      =  67, //video-display
	NETERR_MINOR                 =  68,
	NETERR                       =  69,
	VIPROC                       =  70,
	INPUTS                       =  71,
	DLL                          =  72,
	FUR                          =  73,
	BICOM                        =  74,
	HDFAT                        =  75,
	NET_OVERLOAD                 =  76,
	RCP_BEAT                     =  77,
	LIPSYNC                      =  78,
	AUDIO_IN                     =  79,
	AUDIO_OUT                    =  80,
	FTP_POST                     =  81,
	UPLOAD                       =  82,
	RCFILE                       =  83,
	FSSERVICE                    =  84,
	DHCP                         =  85,
	CGM                          =  86,
	RD_ROM_SYNC                  =  87,
	FRAMEDROP                    =  88,
	ISCSI                        =  89,
	RCP_CMD_GEN                  =  90,
	ISCSI_DISCOVERY              =  91,
	PM_MGR                       =  92,
	EAPOL                        =  93,
	PARSE_EAPOL                  =  94,
	SPANS                        =  95,
	SM_API                       =  96,
	SSL_PRINT                    =  97,
	HD_MGR_DISPATCH              =  98,
	GENLOOP                      =  99,
	CPU_LOAD                     = 100,
	CRYPT_AES                    = 101,
	RTSP                         = 102,
	P_SMTP                       = 103,
	CODEC_DSP                    = 104,
	RULE_ENGINE                  = 105,
	HTTP_CLT                     = 106,
	BOOL_ENGINE                  = 107,
	RCP_CGI                      = 108,
	SUPERVISOR_DBG               = 109,
	HTTP_BEAT                    = 110,
	HTTP_AUTH                    = 111,
	KBD                          = 112,
	SPAN_MGR                     = 113,
	FLASH                        = 114,
	DOME_CTL                     = 115,
	UPNP                         = 116,
	DYNDNS                       = 117,
	UPNPCP                       = 118,
	HDD_REP_OUT                  = 119,
	HDD_REP_IN                   = 120,
	VCA_SOC                      = 121,
	WEB_SERVICE                  = 122,
	NAND_FLASH                   = 123,
	HDD_REP_OUT2                 = 124,
	HDD_REC_OUT                  = 125,
	VDP_PACK                     = 126,
	VDP_UNPACK                   = 127,
	SDCARD                       = 128,
	NTCIP                        = 129,
	REC_PROXY                    = 130,
	SDIO                         = 131,
	VIPROC_ALARM                 = 132,
	MOBILYGEN_SYNC               = 133,
	WLAN_CMD_PRINT               = 134,
	WPA_PRINT                    = 135,
	SENSOR_PRINT                 = 136, //reserved by SH for lowlight  etc.
	RTP_TO_LOCAL                 = 137,
	HDD_REP_UVRM                 = 138,
	AUPROC_ALARM                 = 139,
	VIN_STATE                    = 140,
	CODEC_STATE                  = 141,
	ENCODER_CFG_PATCH            = 142,
	CODEC_WARN                   = 143,
	H264_BITRATE                 = 144,
	HD_MGR                       = 145,
	MP4FILE_CONV                 = 146,
	LLDP                         = 147,
	PARSE_LLDP                   = 148,
	ENC_SILENT                   = 149,
	MEDIA_BACKUP                 = 150,
	HDD_REP_STOP                 = 151,
	BBSS                         = 152,
	ENC_PKT_OUT                  = 153, //packet output debugging
	VOUT                         = 154,
	HEATER_CTRL                  = 155,
	REC_SCHED                    = 156,
	VDP_RECORDER                 = 157,
	RANDGEN                      = 158, //random generator debugging
	ID_CHIP                      = 159, //id chip debugging
	IIC_DBG                      = 160,
	CODEC_RATECTRL               = 161, //short term rate control algorithm
	VALARM                       = 162, //virtual alarm
	FORENSIC_SEARCH              = 163,
	LUA                          = 164,
	TCP_FWD                      = 165,
	CODEC_DPTZ                   = 166, //digtal zoom on driver level
	COPY_JOB                     = 167,
	CODEC_RATEAVG                = 168, //long term rate control/averaging
	EOT_RATE_CTRL                = 169,
	TRANSCODER                   = 170,
	ENC_DELAY                    = 171,
	CODEC_BITSTREAM_FIFO         = 172, //DSP bitstream fifo related msg
	CODEC_CMD                    = 173,
	HDD_PERF_MON                 = 174,
	CODEC_AAA                    = 175,
	FS_PRINT                     = 176,
	STREAM_SEC                   = 177,
	SOCK_KNOCKER                 = 178,
	DROPBOX                      = 179,
	DPTZ_TERM                    = 180, //DPTZ configuration
	ROI_TERM                     = 181, //area-based encoder/quality adj.
	CODEC_ROI                    = 182, //area-based adj.: driver level
	RTCP                         = 183,
	AUTO_TRACKER                 = 184,
	HDD_REP_SM                   = 185,
	HDD_REC_RATE                 = 186,
	TS_FRAME_INFO                = 187,
	REMOTE_DEV_CHECK             = 188,
	SMB                          = 189,
	TCP_ASYNC                    = 190,
	BICOM_WRAPPER                = 191,
	DEV_SIM                      = 192,
	SEI                          = 193,
	FTP_SERVER                   = 194,
	GB_28181                     = 195,
	ALARM_BACKUP                 = 196,
	IMAGEPIPE                    = 197,
	NETSIM                       = 198,
	MALLOCS                      = 199,
	SYSCON                       = 200,
	CODEC_MSG                    = 201,
	CODEC_ENC_MSG                = 202,
	TIMCFW                       = 203,
	ENC_IFRM_OUT                 = 204,
	AUTOFOCUS                    = 205,
	CODEC_OSD                    = 206,
	CODEC_IDR                    = 207,
	CODEC_LOAD                   = 208,
	DEC_LAYOUT                   = 209,
	VINPORT                      = 210,
	SXI_DLL                      = 211,
	CERT_STORE                   = 212,
	VIN_BOARD                    = 213,
	VIN_BOARD_FRAMES             = 214,
	VIN_BOARD_CLOCK              = 215,
	HDD_REP_MV                   = 216,
	SPI_DBG                      = 217,
	IDLL_WRITE                   = 218,
	IDLL_READ                    = 219,
	HDD_REP_FF                   = 220,
	ASCM                         = 221,
	STAMPING                     = 222,
	INST_MENU                    = 223,
	DEWARPING                    = 224,
	CODEC_WARP                   = 225,
	CRYPT_LIB                    = 226,
	CODEC_AAA_DATA               = 227,
	CODEC_AAA_COLLECT            = 228,
	AAA_TERM                     = 229,
	SSH_CLIENT                   = 230,
	CODEC_TICK                   = 231,
	WEB_SOCKETS                  = 232,
	HDD_REP_MSG                  = 233,
	SOCKET_ZSE                   = 234,
	CODEC_ERROR                  = 235,
	ZBAR                         = 236,
	VIDEO_MP4                    = 237,
	XML                          = 238,
	TC_INTEL                     = 239,
	VCD_CACHE                    = 240,
	SCHEDULED_PTZ                = 241,
	PIR_STATE                    = 242,
	CALIBRATION                  = 243,
	IMAGEPIPE_CFG                = 244,
	IMAGEPIPE_CMD                = 245,
	IMAGEPIPE_DEBUG              = 246,
	DRAWING_RAW                  = 247,
	VIDEO_MP4_BUFF               = 248,
	PTZ_CTRL                     = 249,
	CODEC_STATE_UPDT             = 250,
	WLAN_DBG                     = 251,
	CERT_CRL                     = 252,
	ONVIF_SEARCH                 = 253,
	STM32                        = 254,
	SYSLOG_DBG                   = 255,
	HDD_CRYPT                    = 256,
	LED_MGR                      = 257,
	CBS_REST                     = 258,
	SSL_CONN_THREAD              = 259,
	PRINT_IP_V6                  = 260,
	SEQ_VERIFY_OUT               = 261,
	STRATOCAST                   = 262,

	VCS_PRINT_ID_MAX_USED,       // xx TH: has to be the last

	//...
	//...
	//...


	MAX_ID = MAX_HASH_IDS
};

#define MOBILYGEN CODEC_DSP

char *vcs_strncpy(char *to, const char *from, size_t n);
#define STRNCPY vcs_strncpy



//////////////////////////////////////////////////////////////////////////////////
//
// TH: we provide a sprintf function that checks the length
//     use SPRINTF_EX instead of sprintf
//
int vcs_sprintfex(char *buf, size_t max_bytes, const char *fmt, ...);
#define SPRINTF_EX vcs_sprintfex

// Wrapper to use C_PrintOut clas in "C" files
int PO_Print_Internal(char *fmt, ...);
#define PO_Print PO_Print_Internal       // force the app to include this header (else the compiler might not process the "..." arguments right in c-files)
void PO_CrLf(void);
void PO_Return(void);
int PO_SetColumns(int value);
int PO_SetIndent(int value);
int PO_PrintIP(DWORD ip_addr);
int PO_Fill(char c, int len);
void PO_Tab(int abs_columns);
void PO_OpenOutputBuffer(char *buffer, int len, int Flags);
DWORD PO_CloseOutputBuffer(void);


#ifdef __cplusplus
}
#endif


// ------------------------------------------------------------
// print helper classes
// ------------------------------------------------------------
#define BK_COLOR_DFLT      0
#define BK_COLOR_BLACK     40
#define BK_COLOR_RED       41
#define BK_COLOR_GREEN     42
#define BK_COLOR_YELLOW    43
#define BK_COLOR_BLUE      44
#define BK_COLOR_MAGENTA   45
#define BK_COLOR_CYAN      46
#define BK_COLOR_WHITE     47


#define FG_COLOR_DFLT      0
#define FG_COLOR_BLACK     30
#define FG_COLOR_RED       31
#define FG_COLOR_GREEN     32
#define FG_COLOR_BROWN     33
#define FG_COLOR_BLUE      34
#define FG_COLOR_MAGENTA   35
#define FG_COLOR_CYAN      36
#define FG_COLOR_WHITE     37

#define FG_COLOR_GRAY        130
#define FG_COLOR_LIGHT_RED   131
#define FG_COLOR_LIGHT_GREEN 132
#define FG_COLOR_YELLOW      133
#define FG_COLOR_LIGHT_BLUE  134
#define FG_COLOR_PINK        135
#define FG_COLOR_LIGHT_CYAN  136
#define FG_COLOR_LIGHT_GRAY  137


#define PF_COLOR_SUPPORT  (1 << 0)
#define PF_HTML_MODE      (1 << 1)
#define PF_HTML_TXT_MODE ((1 << 2)|PF_HTML_MODE) // HTML "text" output


#ifdef __cplusplus
// ----------------------------------------------


#define PRINT_LINE_BUFF_LEN 512  // maximum bytes can be put with one call of Print()


class C_PrintOut
{
	protected:
      int m_indent;
      int m_MaxColumns;
      int m_CurrColumns;
      BOOL m_CriticalSectionSave;
      BOOL m_LastPrintedCharVisible;
      int m_RightPaddingColumns;
      int m_Flags;
      char* m_ColorTagOpen;

	   virtual void DoIndent(int i);
	   virtual void PrintReformatted(char  *Buffer, int len, BOOL FormattingEnabled);  // attention this changes Buffer content
	   virtual void PrintInternal(char *fmt, ...);
	   virtual void PrintToDevice(char *Buffer, int len) = 0;

	public:

	VCS_DEBUG_MALLOC_OPERATORS

	C_PrintOut *m_ReplacedOutputObject;  // internal use only

	virtual void _InternalPutBuffer(char *buff, int len); // internal use only (changes the buffer content of caller !!!)

	C_PrintOut(int Flags = PF_COLOR_SUPPORT);
	virtual int CriticalSectionSave();                   // Check if Print can be called from a critical section? (Only in buffer mode true)
	virtual void SetRightPadding( int value);            // Print spaces at end of each line until this width is reached (Color formatting)
	virtual void SetColor(int BkColor, int TextColor);   // if PF_COLOR_SUPPORT is set => HTLM and ASCII supports this
	virtual int  Print(char *fmt, ...);                  // Main Print function
	virtual int PrintIP(DWORD ip_addr);
	virtual int Fill(char c, int len);                   // put len times the characted c
	virtual void PrintTags(char *fmt, ...);              // put invisible HTML code (not accounted for formatting: column width and indent)
	virtual void CrLf(int number = 1);                  // Linebreak(s)
	virtual void Return();                               // Only cursor return if not so
	virtual int SetColumns(int value);                   // set number of columns when a linebreak check AFTER each Print() should be performed
	virtual int SetIndent(int value); // absolute        // number of spaces before the first put char (maintained also by auto linebreak)
	virtual int Indent(int value);  // relative          // relative indend + 5 ... - 5 (for cascading in subfunctions)
	virtual void SetFlags(int value) {m_Flags = value;}  // Mode Flags (PF_COLOR_SUPPORT, PF_HTML_MODE)
	virtual void Tab(int col)                            // put spaces until TOTAL number of chars in the curren line are col + indent
	{
	   int PutLeft = col - (m_CurrColumns - m_indent);
	   if (PutLeft > 0)
      {
          DoIndent(PutLeft);
      }
	};
	virtual void OpenForm(char* page, char* method = "get");             // Opens a HTML form
	virtual void CloseForm();                                            // Closes the HTML form
	virtual void AddSubmitButton(char** Args, char** Vals);              // Adds a submit button to the form
	virtual void AddInputField(char* label, char* name, char* val, int size=10, int maxlength=10);              // Adds a input field to the form
	virtual void AddInputFieldMultiple(char* label, char* name, char* val, char* name2, char* val2, int size=10, int maxlength=10);              // Adds a input field to the form
};

// ----------------------------------------------
class C_PrintOutTerm : public C_PrintOut   // print to terminal
{
	protected:
	   char m_LineBuff[256];
	   int m_LineBuffLen;
      DWORD m_DoPrintout;
	   virtual void PrintToDevice(char *Buffer, int len);
	public:

	C_PrintOutTerm(DWORD DoPrintout, int Flags = PF_COLOR_SUPPORT);
	VCS_DEBUG_MALLOC_OPERATORS
};

// ----------------------------------------------
class C_PrintOutBuffered : public C_PrintOut   // print to buffer (e.g. HTML Page)
{
	private:
	  char *m_pbuffer;
	  char *m_pbufWalk;
	  int m_LenLeft;
	  int m_BufferLen;
	  char *m_SelfAllocated;

	protected:

	virtual void PrintToDevice(char *Buffer, int len);

	public:

	C_PrintOutBuffered(char *buffer, int len, int Flags = PF_COLOR_SUPPORT);
	~C_PrintOutBuffered();

	VCS_DEBUG_MALLOC_OPERATORS
	virtual int DataLen();            // Len of current content
	virtual int SpaceLeft() {return m_LenLeft; };
	virtual void ClearBuffer();
	virtual void DumpToTerminal();

};

// ----------------------------------------------

//Helper function to fill debug pages and keep track of the remaining space
static inline void DbgPagePrint(char** buffer, int& remBytes, int& sumBytes, const char* format, ...)
{
	va_list ap;
	va_start(ap, format);
	int len = vcs_vsprintf(*buffer, remBytes, format, ap);
	va_end(ap);

	*buffer  += len;
	remBytes -= len;
	sumBytes += len;
}

#endif //  __cplusplus


#endif //VCS_PRINT_H

